package com.controller.common;

import cn.com.ty.lift.common.utils.ValuePool;
import cn.com.ty.lift.common.verify.Verify;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

/**
 * 文件上传接口
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("common")
public class CommonController {
    private static final String DateInPath = "yyyy/MM/dd";
    private SystemConfiguration systemConfiguration;

    /**
     * 批量上传多个文件
     *
     * @param files 文件列表
     * @return RestResponse
     */
    @PostMapping("uploads")
    public RestResponse uploads(@RequestParam("files") MultipartFile[] files) {
        Verify.notTrue(Objects.isNull(files) || files.length == 0, Verify.Upload.fileDataMissing);
        try {
            Map<String, MultipartFile> fileMap = new HashMap<>();
            //1 先解析文件格式
            for (MultipartFile file : files) {
                String fileName = handleFile(file);
                if (StrUtil.isEmpty(fileName)) {
                    return RestResponse.fail(Verify.Upload.fileFormatNotSupport);
                }
                fileMap.put(fileName, file);
            }
            //2 批量上传
            List<String> urls = new ArrayList<>();
            for (Map.Entry<String, MultipartFile> entry : fileMap.entrySet()) {
                String url = systemConfiguration.build().putObject(entry.getKey(), entry.getValue().getBytes());
                log.info("上传文件，文件URL: {}", url);
                Verify.notNull(url, Verify.Upload.fileUploadFail);
                urls.add(url);
            }
            return RestResponse.success(urls);
        } catch (Exception e) {
            log.error("上传文件异常", e);
            return RestResponse.fail(Verify.Upload.fileUploadFail);
        }
    }

    /**
     * 上传单个文件
     *
     * @param file 上传的文件
     * @return RestResponse
     */
    @PostMapping("upload")
    public RestResponse upload(@RequestParam("file") MultipartFile file) {
        Verify.notTrue(Objects.isNull(file) || file.isEmpty(), Verify.Upload.fileDataMissing);
        try {
            String fileName = handleFile(file);
            if (StrUtil.isEmpty(fileName)) {
                return RestResponse.fail(Verify.Upload.fileFormatNotSupport);
            }
            String url = systemConfiguration.build().putObject(fileName, file.getBytes());
            log.info("上传文件，文件URL: {}", url);
            return RestResponse.success(url);
        } catch (Exception e) {
            log.error("上传文件异常", e);
            return RestResponse.fail(Verify.Upload.fileUploadFail);
        }
    }

    /**
     * 校验文件大小，文件格式，生成新文件名
     *
     * @param file the {@link MultipartFile} to upload
     * @return the name string of the file.
     */
    private String handleFile(MultipartFile file) {
        Verify.notTrue(Objects.isNull(file) || file.isEmpty(), Verify.Upload.fileDataMissing);
        // 获取文件名，带后缀
        String originalFilename = file.getOriginalFilename();
        log.info("上传文件，原文件名：{}", originalFilename);
        // 获取文件的后缀格式
        int lastDotIndex = originalFilename.lastIndexOf(ValuePool.DOT);
        Verify.notTrue(-1 == lastDotIndex, "文件名解析不到文件格式");
        String fileSuffix = originalFilename.substring(lastDotIndex).toLowerCase();
        Verify.notNull(fileSuffix, Verify.Upload.fileFormatIllegal);
        long fileSize = file.getSize();
        if (StrUtil.equalsAny(fileSuffix, Verify.Upload.pics)) {
            Verify.notTrue(fileSize > Verify.Upload.maxSizePic, "文件大小不超过" + Verify.Upload.maxSizePicDesc);
        } else if (StrUtil.equalsAny(fileSuffix, Verify.Upload.files)) {
            Verify.notTrue(fileSize > Verify.Upload.maxSizeFile, "文件大小不超过" + Verify.Upload.maxSizeFileDesc);
        } else if (StrUtil.equalsAny(fileSuffix, Verify.Upload.videos)) {
            Verify.notTrue(fileSize > Verify.Upload.maxSizeVideo, "文件大小不超过" + Verify.Upload.maxSizeVideoDesc);
        } else {
            return null;
        }
        String fileName = StrUtil.format("{}/{}{}", DateUtil.format(DateUtil.date(), DateInPath), IdWorker.getIdStr(), fileSuffix);
        log.info("上传文件，新文件名：{}", fileName);
        return fileName;
    }
}
